# -*- coding: utf-8 -*-

import os
import argparse
from datetime import datetime

from ydl_ai_recommender.src.core.manager import (
    OrderDataManager,
    ChatDataManager,
    UserProfileManager,
    CounselorProfileManager,
    UserEventManager,
)
from ydl_ai_recommender.src.core.indexer import (
    UserCounselorDefaultIndexer,
    UserCounselorOrderIndexer,
    UserCounselorChatIndexer,
    UserCounselorCombinationIndexer,
)
from ydl_ai_recommender.src.data.mysql_client import MySQLClient
from ydl_ai_recommender.src.utils import get_conf_path, get_project_path
from ydl_ai_recommender.src.utils.log import create_logger


logger = create_logger(__name__, 'update.log')


parser = argparse.ArgumentParser(description='壹点灵 咨询师推荐 算法召回 离线更新数据模型')
parser.add_argument(
    '-t', '--task', type=str, required=True,
    choices=('load_db_data', 'make_embedding', 'make_index', 'load_user_event_data'), help='执行任务名称'
)

parser.add_argument(
    '--only_update', default=False, action='store_true', help='从数据库导出数据到本地是否覆盖原数据，默认不覆盖'
)

parser.add_argument('--index_last_date', default=None, type=str, help='构建索引最后日期，超过该日期的数据不使用')
args = parser.parse_args()


def initialize_dir():
    # 创建数据目录
    now = datetime.now()
    really_data_dir = os.path.join(get_project_path(), 'data_{}'.format(now.strftime('%Y%m%d_%H%M%S')))
    default_data_dir = os.path.join(get_project_path(), 'data')

    # 判断data目录是否存在
    if os.path.exists(default_data_dir):
        if os.path.islink(default_data_dir):
            os.unlink(default_data_dir)
        else:
            logger.error('%s 目录已经存在！请备份后删除该目录再重新执行本操作', default_data_dir)

    os.mkdir(really_data_dir)
    logger.info('创建数据保存目录成功')
    
    # 创建软连接
    os.symlink(really_data_dir, default_data_dir)
    logger.info('创建软连接成功 %s -> %s', really_data_dir, default_data_dir)

    # TODO 历史数据删除


if __name__ == '__main__':

    logger.info('')
    if args.task == 'load_db_data':
        
        if args.only_update is False:
            initialize_dir()
        
        logger.info('开始从数据库中更新数据')
        client = MySQLClient.create_from_config_file(get_conf_path())

        managers = [
            ['画像数据', UserProfileManager(client)],
            ['订单数据', OrderDataManager(client)],
            ['询单数据', ChatDataManager(client)],
            ['咨询师画像', CounselorProfileManager(client)],
        ]

        for [name, manager] in managers:
            logger.info('开始更新 %s', name)
            manager.update_data()
            logger.info('%s 更新完成', name)
            logger.info('')

        logger.info('所有数据更新数据完成')

    if args.task == 'make_embedding':
        logger.info('')
        logger.info('--' * 50)
        logger.info('开始构建用户特征 embedding')
        manager = UserProfileManager()
        manager.make_embeddings()
        logger.info('用户特征 embedding 构建完成')

        logger.info('--' * 50)
        logger.info('开始构建咨询师特征 embedding')
        manager = CounselorProfileManager()
        manager.make_embeddings()
        logger.info('咨询师特征 embedding 构建完成')
        

    if args.task == 'make_index':
        indexers = [
            ['[用户->咨询师]兜底关系索引', UserCounselorDefaultIndexer(logger=logger)],
            ['基于订单数据的[用户->咨询师]关系索引', UserCounselorOrderIndexer(logger=logger)],
            ['基于询单数据的[用户->咨询师]关系索引', UserCounselorChatIndexer(logger=logger)],
            ['基于多种数据组合的[用户->咨询师]关系索引', UserCounselorCombinationIndexer(0.8, 0.2, logger=logger)],
        ]

        logger.info('')
        logger.info('--' * 50)

        for [name, indexer] in indexers:
            logger.info('开始构建 %s', name)
            indexer.make_index()
            logger.info('%s 构建完成', name)
            logger.info('')

        logger.info('所有索引更新数据完成')

    if args.task == 'load_user_event_data':
        logger.info('')
        logger.info('开始下载用户埋点数据')
        client = MySQLClient.create_from_config_file(get_conf_path())
        manager = UserEventManager(client)
        # 训练数据
        manager.update_data(start_date='2023-01-01', end_date='2023-02-01')
        # 测试数据
        manager.update_data(start_date='2023-02-01', end_date='2023-02-10')
        logger.info('用户特征虚拟 embedding 构建完成')