# -*- coding: utf-8 -*-

import os
import pickle
from typing import List

from ydl_ai_recommender.src.utils.log import create_logger
from ydl_ai_recommender.src.utils import get_conf_path, get_model_path
from ydl_ai_recommender.src.data.mysql_client import MySQLClientPool
from ydl_ai_recommender.src.core.manager import CounselorProfileManager
from ydl_ai_recommender.src.core.profile import encode_user_profile
from ydl_ai_recommender.src.core.counselor_profile import encode_counselor_profile


class Ranker():
    def __init__(self, is_use_db=True) -> None:
        self.logger = create_logger(__name__, 'ranker.log')
        if is_use_db:
            self.client = MySQLClientPool.create_from_config_file(get_conf_path())
        else:
            self.client = None
            self.logger.warn('未连接数据库')

        self.select_items_str = ', '.join([
            'uid', 'country_code', 'channel_id_type', 'ffrom_login', 'user_preference_cate', 'consult_pay_money',
            'listen_pay_money', 'test_items_pay_money', 'course_pay_money', 'consult_order_num',
            'listen_order_num', 'test_items_order_num', 'course_order_num', 'aidi_cst_bias_city',
            'aidi_cst_bias_sex', 'aidi_cst_bias_price', 'aidi_cst_bias_server_type', 'user_login_city',
            'd30_inquire_order_num', 'd30_session_num',
        ])

        # 用户画像embedding维度，后期需优化
        self.user_embedding_dimension = 98
        self.counselor_embeddings_dimension = 171

        self.counselor_embeddings = self._load_counselor_embeddings()
        model_path = os.path.join(get_model_path(), 'ranker', 'LR_v1.1.pkl')
        with open(model_path, 'rb') as f:
            self.model = pickle.load(f)

    def _load_counselor_embeddings(self):
        counselor_profile_manager = CounselorProfileManager()
        counselor_profiles = counselor_profile_manager.load_xlsx_data('all_counselor_profile.xlsx')
        counselor_embeddings = {}
        for _, profile in counselor_profiles.iterrows():
            counselor_embeddings[str(profile['doctor_id'])] = encode_counselor_profile(profile)
        return counselor_embeddings
        
    def get_user_profile(self, user_id):
        if user_id == '0':
            return None
            
        sql = 'SELECT {} FROM ads.ads_register_user_profiles'.format(self.select_items_str)
        sql += ' WHERE uid={}'.format(user_id)
        try:
            _, all_data = self.client.query(sql)
            if len(all_data) == 0:
                return []
            return all_data[0]
        except Exception as e:
            self.logger.error('获取用户画像数据失败', exc_info=True)
            
        return None

    def default_user_embedding(self):
        return [.0] * self.user_embedding_dimension

    def get_user_embedding(self, user_id):
        user_profile = self.get_user_profile(user_id)
        if not user_profile:
            return self.default_user_embedding()
        
        user_embedding = encode_user_profile(user_profile)
        return user_embedding

    def default_counselor_embedding(self):
        return [.0] * self.counselor_embeddings_dimension
    
    def get_counselor_embeddings(self, counselors: List):
        return [
            self.counselor_embeddings.get(counselor, self.default_counselor_embedding())
            for counselor in counselors
        ]

    def compute_score(self, user_embedding, counselor_embeddings):
        model_input = [
            user_embedding + counselor_embedding
            for counselor_embedding in counselor_embeddings
        ]
        result = self.model.predict_proba(model_input)
        return map(lambda x: x[1] * 100, result)

    def _rank(self, counselors, user_embedding, counselor_embeddings):
        """ 为了方便测试，精排流程拆分为两个函数 """
        scores = self.compute_score(user_embedding, counselor_embeddings)

        ret = []
        for score, counselor in sorted(zip(scores, counselors), reverse=True):
            ret.append({
                'counselor': counselor,
                'score': score,
            })
        return ret

    def rank(self, uid, counselors):
        counselors = counselors.strip()
        if not counselors:
            return []
        
        counselors = counselors.split(',')
        user_embedding = self.get_user_embedding(uid)
        counselor_embeddings = self.get_counselor_embeddings(counselors)
        return self._rank(counselors, user_embedding, counselor_embeddings)


if __name__ == '__main__':
    ranker = Ranker()
    print(ranker.rank('13577448', '17607,4792,4202,13588,11600,13242'))

    print(ranker.rank('0', '17607,4792,4202,13588,11600,0'))